// Copyright (C) 2017 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only

//
//  W A R N I N G
//  -------------
//
// This file is not part of the Qt API.  It exists for the convenience
// of the Network Access API.  This header file may change from
// version to version without notice, or even be removed.
//
// We mean it.
//

#ifndef QABSTRACTOAUTHREPLYHANDLER_P_H
#define QABSTRACTOAUTHREPLYHANDLER_P_H

#include <QtCore/qloggingcategory.h>
#include <QtCore/private/qglobal_p.h>

QT_BEGIN_NAMESPACE

Q_DECLARE_LOGGING_CATEGORY(lcReplyHandler)

QT_END_NAMESPACE

#endif // QABSTRACTOAUTHREPLYHANDLER_P_H
