// Copyright (C) 2024 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only

//
//  W A R N I N G
//  -------------
//
// This file is not part of the Qt API.  It exists for the convenience
// of the Network Access API.  This header file may change from
// version to version without notice, or even be removed.
//
// We mean it.
//

#ifndef QOAUTHOOBREPLYHANDLER_P_H
#define QOAUTHOOBREPLYHANDLER_P_H

#include <private/qobject_p.h>

QT_BEGIN_NAMESPACE

class QOAuthOobReplyHandlerPrivate : public QObjectPrivate
{
};

QT_END_NAMESPACE

#endif // QOAUTHOOBREPLYHANDLER_P_H
